/*
 This file is part of GNU Taler
 (C) 2026 Taler Systems S.A.

 GNU Taler is free software; you can redistribute it and/or modify it under the
 terms of the GNU Affero General Public License as published by the Free Software
 Foundation; either version 3, or (at your option) any later version.

 GNU Taler is distributed in the hope that it will be useful, but WITHOUT ANY
 WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more details.

 You should have received a copy of the GNU Affero General Public License along with
 GNU Anastasis; see the file COPYING.  If not, see <http://www.gnu.org/licenses/>
 */

declare const QR_WIDTH: number;
declare const QR_HEIGHT: number;
declare const I18N_PAYMENT_CONFIRMED_LOADING: string;
declare const I18N_PAYMENT_CONFIRMED_PROBLEM: string;
declare const I18N_PAYMENT_CONFIRMED_ERROR: string;
declare const I18N_PAYMENT_CONFIRMED_NO_ORDER: string;
declare const I18N_PAYMENT_NETWORK_PROBLEM: string;
declare const MERCHANT_BACKEND: string;
declare const MERCHANT_TEMPLATE_ID: string;
declare const MAX_PICKUP_DELAY: number;
declare const TEMPLATE_SUMMARY: string;
declare const TEMPLATE_CHOICES: string;
declare const POLL_WAIT_MS: number;

declare class QRCode {
  constructor(el: HTMLElement | null, options: {
    text: string;
    width: number;
    height: number;
    correctLevel: number;
  });
  static CorrectLevel: {
    L: number;
    M: number;
    Q: number;
    H: number;
  };
}
