/*
     This file is part of GNUnet.
     Copyright (C) 2026 Taler Systems SA

     Paivana is free software; you can redistribute it and/or
     modify it under the terms of the GNU Affero General Public License
     as published by the Free Software Foundation; either version
     3, or (at your option) any later version.

     Paivana is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty
     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
     the GNU Affero General Public License for more details.

     You should have received a copy of the GNU Affero General Public
     License along with Paivana; see the file COPYING.  If not,
     write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

/**
 * @author Christian Grothoff
 * @file paivana-httpd_daemon.h
 * @brief daemon functions
 */
#ifndef PAIVANA_HTTPD_DAEMON_H
#define PAIVANA_HTTPD_DAEMON_H

#include <stdbool.h>
#include <microhttpd.h>
#include <gnunet/gnunet_util_lib.h>


/**
 * Start the HTTP server and begin serving requests.
 */
void
PAIVANA_HTTPD_serve_requests (void);


#endif
